@@ -1,5 +1,5 @@
 /******************************************************************************
- * $Id: shptree.c,v 1.17 2012-01-27 21:09:26 fwarmerdam Exp $
+ * $Id: shptree.c,v 1.20 2018-08-16 15:39:07 erouault Exp $
  *
  * Project:  Shapelib
  * Purpose:  Implementation of quadtree building and searching functions.
@@ -7,13 +7,14 @@
  *
  ******************************************************************************
  * Copyright (c) 1999, Frank Warmerdam
+ * Copyright (c) 2012, Even Rouault <even dot rouault at mines-paris dot org>
  *
  * This software is available under the following "MIT Style" license,
- * or at the option of the licensee under the LGPL (see LICENSE.LGPL).  This
+ * or at the option of the licensee under the LGPL (see COPYING).  This
  * option is discussed in more detail in shapelib.html.
  *
  * --
- * 
+ *
  * Permission is hereby granted, free of charge, to any person obtaining a
  * copy of this software and associated documentation files (the "Software"),
  * to deal in the Software without restriction, including without limitation
@@ -34,6 +35,35 @@
  ******************************************************************************
  *
  * $Log: shptree.c,v $
+ * Revision 1.20  2018-08-16 15:39:07  erouault
+ * * shpopen.c, dbfopen.c, shptree.c, sbnsearch.c: resyc with GDAL
+ * internal shapelib. Mostly to allow building those files as C++
+ * without warning. Also add FTDate entry in DBFFieldType
+ * (see https://github.com/OSGeo/gdal/pull/308). And some other
+ * code cleanups
+ *
+ * Revision 1.19  2016-12-05 12:44:06  erouault
+ * * Major overhaul of Makefile build system to use autoconf/automake.
+ *
+ * * Warning fixes in contrib/
+ *
+ * Revision 1.18  2016-12-04 15:30:15  erouault
+ * * shpopen.c, dbfopen.c, shptree.c, shapefil.h: resync with
+ * GDAL Shapefile driver. Mostly cleanups. SHPObject and DBFInfo
+ * structures extended with new members. New functions:
+ * DBFSetLastModifiedDate, SHPOpenLLEx, SHPRestoreSHX,
+ * SHPSetFastModeReadObject
+ *
+ * * sbnsearch.c: new file to implement original ESRI .sbn spatial
+ * index reading. (no write support). New functions:
+ * SBNOpenDiskTree, SBNCloseDiskTree, SBNSearchDiskTree,
+ * SBNSearchDiskTreeInteger, SBNSearchFreeIds
+ *
+ * * Makefile, makefile.vc, CMakeLists.txt, shapelib.def: updates
+ * with new file and symbols.
+ *
+ * * commit: helper script to cvs commit
+ *
  * Revision 1.17  2012-01-27 21:09:26  fwarmerdam
  * optimize .qix output (gdal #4472)
  *
@@ -93,12 +123,13 @@
 #include <assert.h>
 #include <stdlib.h>
 #include <string.h>
+#include <limits.h>
 
 #ifdef USE_CPL
 #include "cpl_error.h"
 #endif
 
-SHP_CVSID("$Id: shptree.c,v 1.17 2012-01-27 21:09:26 fwarmerdam Exp $")
+SHP_CVSID("$Id: shptree.c,v 1.20 2018-08-16 15:39:07 erouault Exp $")
 
 #ifndef TRUE
 #  define TRUE 1
@@ -118,6 +149,18 @@ static int bBigEndian = 0;
 
 #define SHP_SPLIT_RATIO	0.55
 
+#ifdef __cplusplus
+#define STATIC_CAST(type,x) static_cast<type>(x)
+#define REINTERPRET_CAST(type,x) reinterpret_cast<type>(x)
+#define CONST_CAST(type,x) const_cast<type>(x)
+#define SHPLIB_NULLPTR nullptr
+#else
+#define STATIC_CAST(type,x) ((type)(x))
+#define REINTERPRET_CAST(type,x) ((type)(x))
+#define CONST_CAST(type,x) ((type)(x))
+#define SHPLIB_NULLPTR NULL
+#endif
+
 /************************************************************************/
 /*                             SfRealloc()                              */
 /*                                                                      */
@@ -128,10 +171,10 @@ static int bBigEndian = 0;
 static void * SfRealloc( void * pMem, int nNewSize )
 
 {
-    if( pMem == NULL )
-        return( (void *) malloc(nNewSize) );
+    if( pMem == SHPLIB_NULLPTR )
+        return malloc(nNewSize);
     else
-        return( (void *) realloc(pMem,nNewSize) );
+        return realloc(pMem,nNewSize);
 }
 
 /************************************************************************/
@@ -146,20 +189,20 @@ static SHPTreeNode *SHPTreeNodeCreate( double * padfBoundsMin,
 {
     SHPTreeNode	*psTreeNode;
 
-    psTreeNode = (SHPTreeNode *) malloc(sizeof(SHPTreeNode));
-    if( NULL == psTreeNode )
-        return NULL;
+    psTreeNode = STATIC_CAST(SHPTreeNode *, malloc(sizeof(SHPTreeNode)));
+    if( SHPLIB_NULLPTR == psTreeNode )
+        return SHPLIB_NULLPTR;
 
     psTreeNode->nShapeCount = 0;
-    psTreeNode->panShapeIds = NULL;
-    psTreeNode->papsShapeObj = NULL;
+    psTreeNode->panShapeIds = SHPLIB_NULLPTR;
+    psTreeNode->papsShapeObj = SHPLIB_NULLPTR;
 
     psTreeNode->nSubNodes = 0;
 
-    if( padfBoundsMin != NULL )
+    if( padfBoundsMin != SHPLIB_NULLPTR )
         memcpy( psTreeNode->adfBoundsMin, padfBoundsMin, sizeof(double) * 4 );
 
-    if( padfBoundsMax != NULL )
+    if( padfBoundsMax != SHPLIB_NULLPTR )
         memcpy( psTreeNode->adfBoundsMax, padfBoundsMax, sizeof(double) * 4 );
 
     return psTreeNode;
@@ -177,16 +220,16 @@ SHPTree SHPAPI_CALL1(*)
 {
     SHPTree	*psTree;
 
-    if( padfBoundsMin == NULL && hSHP == NULL )
-        return NULL;
+    if( padfBoundsMin == SHPLIB_NULLPTR && hSHP == SHPLIB_NULLPTR )
+        return SHPLIB_NULLPTR;
 
 /* -------------------------------------------------------------------- */
 /*      Allocate the tree object                                        */
 /* -------------------------------------------------------------------- */
-    psTree = (SHPTree *) malloc(sizeof(SHPTree));
-    if( NULL == psTree )
+    psTree = STATIC_CAST(SHPTree *, malloc(sizeof(SHPTree)));
+    if( SHPLIB_NULLPTR == psTree )
     {
-        return NULL;
+        return SHPLIB_NULLPTR;
     }
 
     psTree->hSHP = hSHP;
@@ -198,12 +241,12 @@ SHPTree SHPAPI_CALL1(*)
 /*      If no max depth was defined, try to select a reasonable one     */
 /*      that implies approximately 8 shapes per node.                   */
 /* -------------------------------------------------------------------- */
-    if( psTree->nMaxDepth == 0 && hSHP != NULL )
+    if( psTree->nMaxDepth == 0 && hSHP != SHPLIB_NULLPTR )
     {
         int	nMaxNodeCount = 1;
         int	nShapeCount;
 
-        SHPGetInfo( hSHP, &nShapeCount, NULL, NULL, NULL );
+        SHPGetInfo( hSHP, &nShapeCount, SHPLIB_NULLPTR, SHPLIB_NULLPTR, SHPLIB_NULLPTR );
         while( nMaxNodeCount*4 < nShapeCount )
         {
             psTree->nMaxDepth += 1;
@@ -236,9 +279,10 @@ SHPTree SHPAPI_CALL1(*)
 /*      Allocate the root node.                                         */
 /* -------------------------------------------------------------------- */
     psTree->psRoot = SHPTreeNodeCreate( padfBoundsMin, padfBoundsMax );
-    if( NULL == psTree->psRoot )
+    if( SHPLIB_NULLPTR == psTree->psRoot )
     {
-        return NULL;
+        free( psTree );
+        return SHPLIB_NULLPTR;
     }
 
 /* -------------------------------------------------------------------- */
@@ -246,37 +290,34 @@ SHPTree SHPAPI_CALL1(*)
 /*      use the bounds of the provided file otherwise the create        */
 /*      function will have already set the bounds.                      */
 /* -------------------------------------------------------------------- */
-    assert( NULL != psTree );
-    assert( NULL != psTree->psRoot );
-	
-    if( padfBoundsMin == NULL )
+    if( padfBoundsMin == SHPLIB_NULLPTR )
     {
-        SHPGetInfo( hSHP, NULL, NULL,
-                    psTree->psRoot->adfBoundsMin, 
+        SHPGetInfo( hSHP, SHPLIB_NULLPTR, SHPLIB_NULLPTR,
+                    psTree->psRoot->adfBoundsMin,
                     psTree->psRoot->adfBoundsMax );
     }
 
 /* -------------------------------------------------------------------- */
-/*      If we have a file, insert all it's shapes into the tree.        */
+/*      If we have a file, insert all its shapes into the tree.        */
 /* -------------------------------------------------------------------- */
-    if( hSHP != NULL )
+    if( hSHP != SHPLIB_NULLPTR )
     {
         int	iShape, nShapeCount;
-        
-        SHPGetInfo( hSHP, &nShapeCount, NULL, NULL, NULL );
+
+        SHPGetInfo( hSHP, &nShapeCount, SHPLIB_NULLPTR, SHPLIB_NULLPTR, SHPLIB_NULLPTR );
 
         for( iShape = 0; iShape < nShapeCount; iShape++ )
         {
             SHPObject	*psShape;
-            
+
             psShape = SHPReadObject( hSHP, iShape );
-            if( psShape != NULL )
+            if( psShape != SHPLIB_NULLPTR )
             {
                 SHPTreeAddShapeId( psTree, psShape );
                 SHPDestroyObject( psShape );
             }
         }
-    }        
+    }
 
     return psTree;
 }
@@ -289,23 +330,23 @@ static void SHPDestroyTreeNode( SHPTreeNode * psTreeNode )
 
 {
     int		i;
-    
-	assert( NULL != psTreeNode );
+
+	assert( SHPLIB_NULLPTR != psTreeNode );
 
     for( i = 0; i < psTreeNode->nSubNodes; i++ )
     {
-        if( psTreeNode->apsSubNode[i] != NULL )
+        if( psTreeNode->apsSubNode[i] != SHPLIB_NULLPTR )
             SHPDestroyTreeNode( psTreeNode->apsSubNode[i] );
     }
-    
-    if( psTreeNode->panShapeIds != NULL )
+
+    if( psTreeNode->panShapeIds != SHPLIB_NULLPTR )
         free( psTreeNode->panShapeIds );
 
-    if( psTreeNode->papsShapeObj != NULL )
+    if( psTreeNode->papsShapeObj != SHPLIB_NULLPTR )
     {
         for( i = 0; i < psTreeNode->nShapeCount; i++ )
         {
-            if( psTreeNode->papsShapeObj[i] != NULL )
+            if( psTreeNode->papsShapeObj[i] != SHPLIB_NULLPTR )
                 SHPDestroyObject( psTreeNode->papsShapeObj[i] );
         }
 
@@ -345,7 +386,7 @@ SHPCheckBoundsOverlap( double * padfBox1Min, double * padfBox1Max,
     {
         if( padfBox2Max[iDim] < padfBox1Min[iDim] )
             return FALSE;
-        
+
         if( padfBox1Max[iDim] < padfBox2Min[iDim] )
             return FALSE;
     }
@@ -366,18 +407,18 @@ static int SHPCheckObjectContained( SHPObject * psObject, int nDimension,
     if( psObject->dfXMin < padfBoundsMin[0]
         || psObject->dfXMax > padfBoundsMax[0] )
         return FALSE;
-    
+
     if( psObject->dfYMin < padfBoundsMin[1]
         || psObject->dfYMax > padfBoundsMax[1] )
         return FALSE;
 
     if( nDimension == 2 )
         return TRUE;
-    
+
     if( psObject->dfZMin < padfBoundsMin[2]
         || psObject->dfZMax > padfBoundsMax[2] )
         return FALSE;
-        
+
     if( nDimension == 3 )
         return TRUE;
 
@@ -395,7 +436,7 @@ static int SHPCheckObjectContained( SHPObject * psObject, int nDimension,
 /*      longest dimension.                                              */
 /************************************************************************/
 
-void SHPAPI_CALL
+static void
 SHPTreeSplitBounds( double *padfBoundsMinIn, double *padfBoundsMaxIn,
                     double *padfBoundsMin1, double * padfBoundsMax1,
                     double *padfBoundsMin2, double * padfBoundsMax2 )
@@ -409,7 +450,7 @@ SHPTreeSplitBounds( double *padfBoundsMinIn, double *padfBoundsMaxIn,
     memcpy( padfBoundsMax1, padfBoundsMaxIn, sizeof(double) * 4 );
     memcpy( padfBoundsMin2, padfBoundsMinIn, sizeof(double) * 4 );
     memcpy( padfBoundsMax2, padfBoundsMaxIn, sizeof(double) * 4 );
-    
+
 /* -------------------------------------------------------------------- */
 /*      Split in X direction.                                           */
 /* -------------------------------------------------------------------- */
@@ -444,9 +485,9 @@ SHPTreeNodeAddShapeId( SHPTreeNode * psTreeNode, SHPObject * psObject,
 
 {
     int		i;
-    
+
 /* -------------------------------------------------------------------- */
-/*      If there are subnodes, then consider wiether this object        */
+/*      If there are subnodes, then consider whether this object        */
 /*      will fit in them.                                               */
 /* -------------------------------------------------------------------- */
     if( nMaxDepth > 1 && psTreeNode->nSubNodes > 0 )
@@ -563,17 +604,17 @@ SHPTreeNodeAddShapeId( SHPTreeNode * psTreeNode, SHPObject * psObject,
 /* -------------------------------------------------------------------- */
     psTreeNode->nShapeCount++;
 
-    psTreeNode->panShapeIds = (int *) 
+    psTreeNode->panShapeIds = STATIC_CAST(int *,
         SfRealloc( psTreeNode->panShapeIds,
-                   sizeof(int) * psTreeNode->nShapeCount );
+                   sizeof(int) * psTreeNode->nShapeCount ));
     psTreeNode->panShapeIds[psTreeNode->nShapeCount-1] = psObject->nShapeId;
 
-    if( psTreeNode->papsShapeObj != NULL )
+    if( psTreeNode->papsShapeObj != SHPLIB_NULLPTR )
     {
-        psTreeNode->papsShapeObj = (SHPObject **)
+        psTreeNode->papsShapeObj = STATIC_CAST(SHPObject **,
             SfRealloc( psTreeNode->papsShapeObj,
-                       sizeof(void *) * psTreeNode->nShapeCount );
-        psTreeNode->papsShapeObj[psTreeNode->nShapeCount-1] = NULL;
+                       sizeof(void *) * psTreeNode->nShapeCount ));
+        psTreeNode->papsShapeObj[psTreeNode->nShapeCount-1] = SHPLIB_NULLPTR;
     }
 
     return TRUE;
@@ -603,7 +644,7 @@ SHPTreeAddShapeId( SHPTree * psTree, SHPObject * psObject )
 /*      tree node by tree node basis.                                   */
 /************************************************************************/
 
-void SHPAPI_CALL
+static void
 SHPTreeCollectShapeIds( SHPTree *hTree, SHPTreeNode * psTreeNode,
                         double * padfBoundsMin, double * padfBoundsMax,
                         int * pnShapeCount, int * pnMaxShapes,
@@ -611,7 +652,7 @@ SHPTreeCollectShapeIds( SHPTree *hTree, SHPTreeNode * psTreeNode,
 
 {
     int		i;
-    
+
 /* -------------------------------------------------------------------- */
 /*      Does this node overlap the area of interest at all?  If not,    */
 /*      return without adding to the list at all.                       */
@@ -629,8 +670,8 @@ SHPTreeCollectShapeIds( SHPTree *hTree, SHPTreeNode * psTreeNode,
     if( *pnShapeCount + psTreeNode->nShapeCount > *pnMaxShapes )
     {
         *pnMaxShapes = (*pnShapeCount + psTreeNode->nShapeCount) * 2 + 20;
-        *ppanShapeList = (int *)
-            SfRealloc(*ppanShapeList,sizeof(int) * *pnMaxShapes);
+        *ppanShapeList = STATIC_CAST(int *,
+            SfRealloc(*ppanShapeList,sizeof(int) * *pnMaxShapes));
     }
 
 /* -------------------------------------------------------------------- */
@@ -640,13 +681,13 @@ SHPTreeCollectShapeIds( SHPTree *hTree, SHPTreeNode * psTreeNode,
     {
         (*ppanShapeList)[(*pnShapeCount)++] = psTreeNode->panShapeIds[i];
     }
-    
+
 /* -------------------------------------------------------------------- */
 /*      Recurse to subnodes if they exist.                              */
 /* -------------------------------------------------------------------- */
     for( i = 0; i < psTreeNode->nSubNodes; i++ )
     {
-        if( psTreeNode->apsSubNode[i] != NULL )
+        if( psTreeNode->apsSubNode[i] != SHPLIB_NULLPTR )
             SHPTreeCollectShapeIds( hTree, psTreeNode->apsSubNode[i],
                                     padfBoundsMin, padfBoundsMax,
                                     pnShapeCount, pnMaxShapes,
@@ -668,7 +709,7 @@ SHPTreeCollectShapeIds( SHPTree *hTree, SHPTreeNode * psTreeNode,
 static int
 compare_ints( const void * a, const void * b)
 {
-    return (*(int*)a) - (*(int*)b);
+    return *REINTERPRET_CAST(const int*, a) - *REINTERPRET_CAST(const int*, b);
 }
 
 int SHPAPI_CALL1(*)
@@ -677,7 +718,7 @@ SHPTreeFindLikelyShapes( SHPTree * hTree,
                          int * pnShapeCount )
 
 {
-    int	*panShapeList=NULL, nMaxShapes = 0;
+    int	*panShapeList=SHPLIB_NULLPTR, nMaxShapes = 0;
 
 /* -------------------------------------------------------------------- */
 /*      Perform the search by recursive descent.                        */
@@ -693,7 +734,8 @@ SHPTreeFindLikelyShapes( SHPTree * hTree,
 /*      Sort the id array                                               */
 /* -------------------------------------------------------------------- */
 
-    qsort(panShapeList, *pnShapeCount, sizeof(int), compare_ints);
+    if( panShapeList != SHPLIB_NULLPTR )
+        qsort(panShapeList, *pnShapeCount, sizeof(int), compare_ints);
 
     return panShapeList;
 }
@@ -701,7 +743,7 @@ SHPTreeFindLikelyShapes( SHPTree * hTree,
 /************************************************************************/
 /*                          SHPTreeNodeTrim()                           */
 /*                                                                      */
-/*      This is the recurve version of SHPTreeTrimExtraNodes() that     */
+/*      This is the recursive version of SHPTreeTrimExtraNodes() that   */
 /*      walks the tree cleaning it up.                                  */
 /************************************************************************/
 
@@ -741,9 +783,9 @@ static int SHPTreeNodeTrim( SHPTreeNode * psTreeNode )
         memcpy(psTreeNode->adfBoundsMax, psSubNode->adfBoundsMax,
                sizeof(psSubNode->adfBoundsMax));
         psTreeNode->nShapeCount = psSubNode->nShapeCount;
-        assert(psTreeNode->panShapeIds == NULL);
+        assert(psTreeNode->panShapeIds == SHPLIB_NULLPTR);
         psTreeNode->panShapeIds = psSubNode->panShapeIds;
-        assert(psTreeNode->papsShapeObj == NULL);
+        assert(psTreeNode->papsShapeObj == SHPLIB_NULLPTR);
         psTreeNode->papsShapeObj = psSubNode->papsShapeObj;
         psTreeNode->nSubNodes = psSubNode->nSubNodes;
         for( i = 0; i < psSubNode->nSubNodes; i++ )
@@ -777,7 +819,7 @@ SHPTreeTrimExtraNodes( SHPTree * hTree )
 /*      Swap a 2, 4 or 8 byte word.                                     */
 /************************************************************************/
 
-static void SwapWord( int length, void * wordP )
+static void	SwapWord( int length, void * wordP )
 
 {
     int		i;
@@ -785,9 +827,9 @@ static void SwapWord( int length, void * wordP )
 
     for( i=0; i < length/2; i++ )
     {
-	temp = ((unsigned char *) wordP)[i];
-	((unsigned char *)wordP)[i] = ((unsigned char *) wordP)[length-i-1];
-	((unsigned char *) wordP)[length-i-1] = temp;
+	temp = STATIC_CAST(unsigned char*, wordP)[i];
+	STATIC_CAST(unsigned char*, wordP)[i] = STATIC_CAST(unsigned char*, wordP)[length-i-1];
+	STATIC_CAST(unsigned char*, wordP)[length-i-1] = temp;
     }
 }
 
@@ -807,18 +849,18 @@ SHPTreeDiskHandle SHPOpenDiskTree( const char* pszQIXFilename,
 {
     SHPTreeDiskHandle hDiskTree;
 
-    hDiskTree = (SHPTreeDiskHandle) calloc(sizeof(struct SHPDiskTreeInfo),1);
+    hDiskTree = STATIC_CAST(SHPTreeDiskHandle, calloc(sizeof(struct SHPDiskTreeInfo),1));
 
-    if (psHooks == NULL)
+    if (psHooks == SHPLIB_NULLPTR)
         SASetupDefaultHooks( &(hDiskTree->sHooks) );
     else
         memcpy( &(hDiskTree->sHooks), psHooks, sizeof(SAHooks) );
 
     hDiskTree->fpQIX = hDiskTree->sHooks.FOpen(pszQIXFilename, "rb");
-    if (hDiskTree->fpQIX == NULL)
+    if (hDiskTree->fpQIX == SHPLIB_NULLPTR)
     {
         free(hDiskTree);
-        return NULL;
+        return SHPLIB_NULLPTR;
     }
 
     return hDiskTree;
@@ -830,7 +872,7 @@ SHPTreeDiskHandle SHPOpenDiskTree( const char* pszQIXFilename,
 
 void SHPCloseDiskTree( SHPTreeDiskHandle hDiskTree )
 {
-    if (hDiskTree == NULL)
+    if (hDiskTree == SHPLIB_NULLPTR)
         return;
 
     hDiskTree->sHooks.FClose(hDiskTree->fpQIX);
@@ -843,23 +885,24 @@ void SHPCloseDiskTree( SHPTreeDiskHandle hDiskTree )
 
 static int
 SHPSearchDiskTreeNode( SHPTreeDiskHandle hDiskTree, double *padfBoundsMin, double *padfBoundsMax,
-                       int **ppanResultBuffer, int *pnBufferMax, 
-                       int *pnResultCount, int bNeedSwap )
+                       int **ppanResultBuffer, int *pnBufferMax,
+                       int *pnResultCount, int bNeedSwap, int nRecLevel )
 
 {
-    int i;
-    int offset;
-    int numshapes, numsubnodes;
+    unsigned int i;
+    unsigned int offset;
+    unsigned int numshapes, numsubnodes;
     double adfNodeBoundsMin[2], adfNodeBoundsMax[2];
+    int nFReadAcc;
 
 /* -------------------------------------------------------------------- */
 /*      Read and unswap first part of node info.                        */
 /* -------------------------------------------------------------------- */
-    hDiskTree->sHooks.FRead( &offset, 4, 1, hDiskTree->fpQIX );
+    nFReadAcc = STATIC_CAST(int, hDiskTree->sHooks.FRead( &offset, 4, 1, hDiskTree->fpQIX ));
     if ( bNeedSwap ) SwapWord ( 4, &offset );
 
-    hDiskTree->sHooks.FRead( adfNodeBoundsMin, sizeof(double), 2, hDiskTree->fpQIX );
-    hDiskTree->sHooks.FRead( adfNodeBoundsMax, sizeof(double), 2, hDiskTree->fpQIX );
+    nFReadAcc += STATIC_CAST(int, hDiskTree->sHooks.FRead( adfNodeBoundsMin, sizeof(double), 2, hDiskTree->fpQIX ));
+    nFReadAcc += STATIC_CAST(int, hDiskTree->sHooks.FRead( adfNodeBoundsMax, sizeof(double), 2, hDiskTree->fpQIX ));
     if ( bNeedSwap )
     {
         SwapWord( 8, adfNodeBoundsMin + 0 );
@@ -867,15 +910,36 @@ SHPSearchDiskTreeNode( SHPTreeDiskHandle hDiskTree, double *padfBoundsMin, doubl
         SwapWord( 8, adfNodeBoundsMax + 0 );
         SwapWord( 8, adfNodeBoundsMax + 1 );
     }
-      
-    hDiskTree->sHooks.FRead( &numshapes, 4, 1, hDiskTree->fpQIX );
+
+    nFReadAcc += STATIC_CAST(int, hDiskTree->sHooks.FRead( &numshapes, 4, 1, hDiskTree->fpQIX ));
     if ( bNeedSwap ) SwapWord ( 4, &numshapes );
 
+    /* Check that we could read all previous values */
+    if( nFReadAcc != 1 + 2 + 2 + 1 )
+    {
+        hDiskTree->sHooks.Error("I/O error");
+        return FALSE;
+    }
+
+    /* Sanity checks to avoid int overflows in later computation */
+    if( offset > INT_MAX - sizeof(int) )
+    {
+        hDiskTree->sHooks.Error("Invalid value for offset");
+        return FALSE;
+    }
+
+    if( numshapes > (INT_MAX - offset - sizeof(int)) / sizeof(int) ||
+        numshapes > INT_MAX / sizeof(int) - *pnResultCount )
+    {
+        hDiskTree->sHooks.Error("Invalid value for numshapes");
+        return FALSE;
+    }
+
 /* -------------------------------------------------------------------- */
 /*      If we don't overlap this node at all, we can just fseek()       */
 /*      pass this node info and all subnodes.                           */
 /* -------------------------------------------------------------------- */
-    if( !SHPCheckBoundsOverlap( adfNodeBoundsMin, adfNodeBoundsMax, 
+    if( !SHPCheckBoundsOverlap( adfNodeBoundsMin, adfNodeBoundsMax,
                                 padfBoundsMin, padfBoundsMax, 2 ) )
     {
         offset += numshapes*sizeof(int) + sizeof(int);
@@ -886,17 +950,35 @@ SHPSearchDiskTreeNode( SHPTreeDiskHandle hDiskTree, double *padfBoundsMin, doubl
 /* -------------------------------------------------------------------- */
 /*      Add all the shapeids at this node to our list.                  */
 /* -------------------------------------------------------------------- */
-    if(numshapes > 0) 
+    if(numshapes > 0)
     {
-        if( *pnResultCount + numshapes > *pnBufferMax )
+        if( *pnResultCount + numshapes > STATIC_CAST(unsigned int, *pnBufferMax) )
         {
-            *pnBufferMax = (int) ((*pnResultCount + numshapes + 100) * 1.25);
-            *ppanResultBuffer = (int *) 
-                SfRealloc( *ppanResultBuffer, *pnBufferMax * sizeof(int) );
+            int* pNewBuffer;
+
+            *pnBufferMax = (*pnResultCount + numshapes + 100) * 5 / 4;
+
+            if( STATIC_CAST(size_t, *pnBufferMax) > INT_MAX / sizeof(int) )
+                *pnBufferMax = *pnResultCount + numshapes;
+
+            pNewBuffer = STATIC_CAST(int *,
+                SfRealloc( *ppanResultBuffer, *pnBufferMax * sizeof(int) ));
+
+            if( pNewBuffer == SHPLIB_NULLPTR )
+            {
+                hDiskTree->sHooks.Error("Out of memory error");
+                return FALSE;
+            }
+
+            *ppanResultBuffer = pNewBuffer;
         }
 
-        hDiskTree->sHooks.FRead( *ppanResultBuffer + *pnResultCount, 
-               sizeof(int), numshapes, hDiskTree->fpQIX );
+        if( hDiskTree->sHooks.FRead( *ppanResultBuffer + *pnResultCount,
+                    sizeof(int), numshapes, hDiskTree->fpQIX ) != numshapes )
+        {
+            hDiskTree->sHooks.Error("I/O error");
+            return FALSE;
+        }
 
         if (bNeedSwap )
         {
@@ -904,20 +986,29 @@ SHPSearchDiskTreeNode( SHPTreeDiskHandle hDiskTree, double *padfBoundsMin, doubl
                 SwapWord( 4, *ppanResultBuffer + *pnResultCount + i );
         }
 
-        *pnResultCount += numshapes; 
-    } 
+        *pnResultCount += numshapes;
+    }
 
 /* -------------------------------------------------------------------- */
 /*      Process the subnodes.                                           */
 /* -------------------------------------------------------------------- */
-    hDiskTree->sHooks.FRead( &numsubnodes, 4, 1, hDiskTree->fpQIX );
+    if( hDiskTree->sHooks.FRead( &numsubnodes, 4, 1, hDiskTree->fpQIX ) != 1 )
+    {
+        hDiskTree->sHooks.Error("I/O error");
+        return FALSE;
+    }
     if ( bNeedSwap  ) SwapWord ( 4, &numsubnodes );
+    if( numsubnodes > 0 && nRecLevel == 32 )
+    {
+        hDiskTree->sHooks.Error("Shape tree is too deep");
+        return FALSE;
+    }
 
     for(i=0; i<numsubnodes; i++)
     {
-        if( !SHPSearchDiskTreeNode( hDiskTree, padfBoundsMin, padfBoundsMax, 
-                                    ppanResultBuffer, pnBufferMax, 
-                                    pnResultCount, bNeedSwap ) )
+        if( !SHPSearchDiskTreeNode( hDiskTree, padfBoundsMin, padfBoundsMax,
+                                    ppanResultBuffer, pnBufferMax,
+                                    pnResultCount, bNeedSwap, nRecLevel + 1 ) )
             return FALSE;
     }
 
@@ -932,8 +1023,9 @@ static
 SAOffset SHPTreeReadLibc( void *p, SAOffset size, SAOffset nmemb, SAFile file )
 
 {
-    return (SAOffset) fread( p, (size_t) size, (size_t) nmemb,
-                                 (FILE *) file );
+    return STATIC_CAST(SAOffset, fread( p, STATIC_CAST(size_t, size),
+                             STATIC_CAST(size_t, nmemb),
+                             REINTERPRET_CAST(FILE*, file) ));
 }
 
 /************************************************************************/
@@ -944,15 +1036,16 @@ static
 SAOffset SHPTreeSeekLibc( SAFile file, SAOffset offset, int whence )
 
 {
-    return (SAOffset) fseek( (FILE *) file, (long) offset, whence );
+    return STATIC_CAST(SAOffset, fseek( REINTERPRET_CAST(FILE*, file),
+                             STATIC_CAST(long, offset), whence ));
 }
 
 /************************************************************************/
 /*                         SHPSearchDiskTree()                          */
 /************************************************************************/
 
-int SHPAPI_CALL1(*) 
-SHPSearchDiskTree( FILE *fp, 
+int SHPAPI_CALL1(*)
+SHPSearchDiskTree( FILE *fp,
                    double *padfBoundsMin, double *padfBoundsMax,
                    int *pnShapeCount )
 {
@@ -964,7 +1057,7 @@ SHPSearchDiskTree( FILE *fp,
     sDiskTree.sHooks.FSeek = SHPTreeSeekLibc;
     sDiskTree.sHooks.FRead = SHPTreeReadLibc;
 
-    sDiskTree.fpQIX = (SAFile)fp;
+    sDiskTree.fpQIX = REINTERPRET_CAST(SAFile, fp);
 
     return SHPSearchDiskTreeEx( &sDiskTree, padfBoundsMin, padfBoundsMax,
                                  pnShapeCount );
@@ -974,14 +1067,14 @@ SHPSearchDiskTree( FILE *fp,
 /*                       SHPSearchDiskTreeEx()                         */
 /************************************************************************/
 
-int* SHPSearchDiskTreeEx( SHPTreeDiskHandle hDiskTree, 
-                     double *padfBoundsMin, double *padfBoundsMax,
-                     int *pnShapeCount )
+int* SHPSearchDiskTreeEx( SHPTreeDiskHandle hDiskTree,
+                          double *padfBoundsMin, double *padfBoundsMax,
+                          int *pnShapeCount )
 
 {
     int i, bNeedSwap, nBufferMax = 0;
     unsigned char abyBuf[16];
-    int *panResultBuffer = NULL;
+    int *panResultBuffer = SHPLIB_NULLPTR;
 
     *pnShapeCount = 0;
 
@@ -989,7 +1082,7 @@ int* SHPSearchDiskTreeEx( SHPTreeDiskHandle hDiskTree,
 /*	Establish the byte order on this machine.	  	        */
 /* -------------------------------------------------------------------- */
     i = 1;
-    if( *((unsigned char *) &i) == 1 )
+    if( *REINTERPRET_CAST(unsigned char *, &i) == 1 )
         bBigEndian = FALSE;
     else
         bBigEndian = TRUE;
@@ -1001,7 +1094,7 @@ int* SHPSearchDiskTreeEx( SHPTreeDiskHandle hDiskTree,
     hDiskTree->sHooks.FRead( abyBuf, 16, 1, hDiskTree->fpQIX );
 
     if( memcmp( abyBuf, "SQT", 3 ) != 0 )
-        return NULL;
+        return SHPLIB_NULLPTR;
 
     if( (abyBuf[3] == 2 && bBigEndian)
         || (abyBuf[3] == 1 && !bBigEndian) )
@@ -1010,22 +1103,28 @@ int* SHPSearchDiskTreeEx( SHPTreeDiskHandle hDiskTree,
         bNeedSwap = TRUE;
 
 /* -------------------------------------------------------------------- */
-/*      Search through root node and it's decendents.                   */
+/*      Search through root node and its descendants.                   */
 /* -------------------------------------------------------------------- */
-    if( !SHPSearchDiskTreeNode( hDiskTree, padfBoundsMin, padfBoundsMax, 
-                                &panResultBuffer, &nBufferMax, 
-                                pnShapeCount, bNeedSwap ) )
+    if( !SHPSearchDiskTreeNode( hDiskTree, padfBoundsMin, padfBoundsMax,
+                                &panResultBuffer, &nBufferMax,
+                                pnShapeCount, bNeedSwap, 0 ) )
     {
-        if( panResultBuffer != NULL )
+        if( panResultBuffer != SHPLIB_NULLPTR )
             free( panResultBuffer );
         *pnShapeCount = 0;
-        return NULL;
+        return SHPLIB_NULLPTR;
     }
 /* -------------------------------------------------------------------- */
 /*      Sort the id array                                               */
 /* -------------------------------------------------------------------- */
-    qsort(panResultBuffer, *pnShapeCount, sizeof(int), compare_ints);
-    
+
+    /* To distinguish between empty intersection from error case */
+    if( panResultBuffer == SHPLIB_NULLPTR )
+        panResultBuffer = STATIC_CAST(int*, calloc(1, sizeof(int)));
+    else
+        qsort(panResultBuffer, *pnShapeCount, sizeof(int), compare_ints);
+
+
     return panResultBuffer;
 }
 
@@ -1037,16 +1136,16 @@ int* SHPSearchDiskTreeEx( SHPTreeDiskHandle hDiskTree,
 /*      seek past them all efficiently.                                 */
 /************************************************************************/
 
-static int SHPGetSubNodeOffset( SHPTreeNode *node) 
+static int SHPGetSubNodeOffset( SHPTreeNode *node)
 {
     int i;
-    long offset=0;
+    int offset=0;
 
-    for(i=0; i<node->nSubNodes; i++ ) 
+    for(i=0; i<node->nSubNodes; i++ )
     {
-        if(node->apsSubNode[i]) 
+        if(node->apsSubNode[i])
         {
-            offset += 4*sizeof(double) 
+            offset += 4*sizeof(double)
                 + (node->apsSubNode[i]->nShapeCount+3)*sizeof(int);
             offset += SHPGetSubNodeOffset(node->apsSubNode[i]);
         }
@@ -1059,19 +1158,19 @@ static int SHPGetSubNodeOffset( SHPTreeNode *node)
 /*                          SHPWriteTreeNode()                          */
 /************************************************************************/
 
-static void SHPWriteTreeNode( SAFile fp, SHPTreeNode *node, SAHooks* psHooks) 
+static void SHPWriteTreeNode( SAFile fp, SHPTreeNode *node, SAHooks* psHooks)
 {
     int i,j;
     int offset;
-    unsigned char *pabyRec = NULL;
-    assert( NULL != node );
+    unsigned char *pabyRec;
+    assert( SHPLIB_NULLPTR != node );
 
     offset = SHPGetSubNodeOffset(node);
-  
-    pabyRec = (unsigned char *) 
+
+    pabyRec = STATIC_CAST(unsigned char *,
         malloc(sizeof(double) * 4
-               + (3 * sizeof(int)) + (node->nShapeCount * sizeof(int)) );
-    if( NULL == pabyRec )
+               + (3 * sizeof(int)) + (node->nShapeCount * sizeof(int)) ));
+    if( SHPLIB_NULLPTR == pabyRec )
     {
 #ifdef USE_CPL
         CPLError( CE_Fatal, CPLE_OutOfMemory, "Memory allocation failure");
@@ -1090,13 +1189,14 @@ static void SHPWriteTreeNode( SAFile fp, SHPTreeNode *node, SAHooks* psHooks)
 
     memcpy( pabyRec+36, &node->nShapeCount, 4);
     j = node->nShapeCount * sizeof(int);
-    memcpy( pabyRec+40, node->panShapeIds, j);
+    if( j )
+        memcpy( pabyRec+40, node->panShapeIds, j);
     memcpy( pabyRec+j+40, &node->nSubNodes, 4);
 
     psHooks->FWrite( pabyRec, 44+j, 1, fp );
     free (pabyRec);
-  
-    for(i=0; i<node->nSubNodes; i++ ) 
+
+    for(i=0; i<node->nSubNodes; i++ )
     {
         if(node->apsSubNode[i])
             SHPWriteTreeNode( fp, node->apsSubNode[i], psHooks);
@@ -1128,17 +1228,17 @@ int SHPWriteTreeLL(SHPTree *tree, const char *filename, SAHooks* psHooks )
     SAFile              fp;
 
     SAHooks sHooks;
-    if (psHooks == NULL)
+    if (psHooks == SHPLIB_NULLPTR)
     {
         SASetupDefaultHooks( &sHooks );
         psHooks = &sHooks;
     }
-  
+
 /* -------------------------------------------------------------------- */
 /*      Open the output file.                                           */
 /* -------------------------------------------------------------------- */
     fp = psHooks->FOpen(filename, "wb");
-    if( fp == NULL ) 
+    if( fp == SHPLIB_NULLPTR )
     {
         return FALSE;
     }
@@ -1147,16 +1247,16 @@ int SHPWriteTreeLL(SHPTree *tree, const char *filename, SAHooks* psHooks )
 /*	Establish the byte order on this machine.	  	        */
 /* -------------------------------------------------------------------- */
     i = 1;
-    if( *((unsigned char *) &i) == 1 )
+    if( *REINTERPRET_CAST(unsigned char *, &i) == 1 )
         bBigEndian = FALSE;
     else
         bBigEndian = TRUE;
-  
+
 /* -------------------------------------------------------------------- */
 /*      Write the header.                                               */
 /* -------------------------------------------------------------------- */
     memcpy( abyBuf+0, signature, 3 );
-    
+
     if( bBigEndian )
         abyBuf[3] = 2; /* New MSB */
     else
@@ -1179,8 +1279,8 @@ int SHPWriteTreeLL(SHPTree *tree, const char *filename, SAHooks* psHooks )
 /*      Write all the nodes "in order".                                 */
 /* -------------------------------------------------------------------- */
 
-    SHPWriteTreeNode( fp, tree->psRoot, psHooks );  
-    
+    SHPWriteTreeNode( fp, tree->psRoot, psHooks );
+
     psHooks->FClose( fp );
 
     return TRUE;
